import QtQuick 1.0

Item {
    id: infoBubble

    property variant mapView

    property bool open: false
    property bool forceAnimation: false
    property bool fingerDownOnInfoBubble: false

    property int _MIN_WIDTH_: 180
    property int _MAX_WIDTH_: 342

    property int _ICON_WIDTH_: 78
    property int _ICON_HEIGHT_: 67

    property variant content: Rectangle {
        color: "transparent"
    }
    property alias _content: internals.currentContent
    property variant position: null

    property bool clickEffectEnabled: !mapView.getBehavior(mapView._NO_INFO_BUBBLE_CLICK_EFFECT_)

    property string imageUrl
    onImageUrlChanged: {
        bubbleIcon.source = imageUrl || mapAssetsPath + "images/layout/poiGeneric.png"
    }
    property bool imageVisible: false

    property bool contentFillsCompletely: false
    property int forceWidth: 0
    onForceWidthChanged: {
        internals.resetContent();
    }

    property int _width: bubbleBody.width
    property int _height: bubbleBody.height
    property int _x: bubbleBody.x
    property int _y: bubbleBody.y

    signal infoBubbleClicked();
    signal infoBubbleDoubleClicked();
	signal infoBubbleClosed();
	signal infoBubbleLongPressed();
	signal infoBubbleVisibilityChanged();

    Item {
        id: internals

        property variant contentToApply: null
        property variant currentContent: null

        function applyContent() {
            if (internals.contentToApply) {
                var content = internals.contentToApply;
                internals.contentToApply = null;

                internals.removeCurrentContent();
                internals.currentContent = content;

                internals.resetContent();
            }
        }

        function resetContent() {
            var content = internals.currentContent;
            if (content) {
                visible = true;
                content.parent = bubbleBodyContent;
                content.visible = true;

                var widthToApply = content.width +
                    (
                        infoBubble.imageVisible?
                            infoBubble._ICON_WIDTH_
                          :
                            (bubbleBodyContent.anchors.leftMargin + bubbleBodyContent.anchors.rightMargin)
                    ) +
                    bubbleBodyContent.anchors.leftMargin + bubbleBodyContent.anchors.rightMargin
                ;
                if (widthToApply < infoBubble._MIN_WIDTH_) {
                    widthToApply = infoBubble._MIN_WIDTH_;
                } else if (widthToApply > (infoBubble.forceWidth || infoBubble._MAX_WIDTH_)) {
                    widthToApply = infoBubble.forceWidth || infoBubble._MAX_WIDTH_;
                }

                bubbleBody.width = widthToApply;
                bubbleBody.height = content.height + bubbleBodyContent.anchors.topMargin + bubbleBodyContent.anchors.bottomMargin;
            }
        }

        function removeCurrentContent() {
            if (internals.currentContent) {
                internals.currentContent.visible = false;
                if (internals.currentContent.destroy) {
                    internals.currentContent.destroy();
                }
            }
            internals.currentContent = null;
        }
    }

	onContentChanged: {
        internals.removeCurrentContent();
        infoBubble.contentFillsCompletely = false;
        infoBubble.forceWidth = 0;
        infoBubble.forceAnimation = false;
        if (content) {
            internals.contentToApply = content;
            if (!internals.currentContent) {
                internals.applyContent();
            }
        }
	}

    Timer {
        interval: 550

        running: infoBubble.open
        repeat: true

        onTriggered: { internals.applyContent() }
    }

	states: [
		State {
			name: "open";
			when: infoBubble.open
		},
		State {
			name: "close";
			when: !infoBubble.open
		}
	]

	transitions: [
		Transition {
			from: "close"
			to: "open"

            SequentialAnimation {
				ParallelAnimation{
					PropertyAnimation {
						target: bubbleBody;
						property: "scale";
                        easing.period: 1
                        easing.type: Easing.OutElastic; from: 0; to: 1; duration: infoBubble.forceAnimation? .01 : 400;
					}
					PropertyAnimation {
						target: bubblePeak;
						property: "opacity";
                        easing.type: Easing.Linear; from: 0; to: 1.0; duration: infoBubble.forceAnimation? .01 : 200;
					}
                }
				ScriptAction {
					script: {
                        if (!open) {
                            imageUrl = "";
                        }
						infoBubble.infoBubbleVisibilityChanged();

                        infoBubble.forceAnimation = false;
					}
				}
			}
		},
		Transition {
			from: "open"
			to: "close"

            SequentialAnimation {
				ParallelAnimation{
					PropertyAnimation {
						target: bubbleBody;
						property: "scale";
                        easing.period: 1
                        easing.type: Easing.InElastic; from: 1; to: 0; duration: infoBubble.forceAnimation? .01 : 400;
					}
					PropertyAnimation {
						target: bubblePeak;
						property: "opacity";
                        easing.type: Easing.Linear; from: 1.0; to: 0; duration: infoBubble.forceAnimation? .01 : 200;
					}
                }
				ScriptAction {
					script: {
                        imageUrl = "";
						infoBubble.infoBubbleVisibilityChanged();
                        bubbleBody.width = 180;

                        internals.removeCurrentContent();

                        infoBubble.forceAnimation = false;
					}
				}
			}
		}
	]


    Item {
        id: bubbleBody

        width: 180
        height: 71
        scale: 0

        x: -(width / 2)
        y: -(height + 30)

        Rectangle {
            opacity: 0.8
            color: {
                var schemeParts = new String(mapView.scheme).split(".");
                return schemeParts[0] == "hybrid" || schemeParts[1] == "night"? "white" : "black";
            }
            radius: 16

            anchors.fill: parent
        }

        Rectangle {
            opacity: 0.8
            color: "black"
            radius: 16

            anchors.fill: parent
            anchors.topMargin: 1
            anchors.leftMargin: 1
            anchors.rightMargin: 1
        }

        Rectangle {
            id: hoverRectangle

            color: "white"
            opacity: 0.2
            radius: 16

            visible: false

            anchors.fill: parent
        }

        Image {
            id: bubbleIcon
            source: mapAssetsPath + "images/layout/poiGeneric.png"

            visible: false && infoBubble.imageVisible

            width: infoBubble._ICON_WIDTH_ * ((visible)? 1 : 0)
            height: infoBubble._ICON_HEIGHT_ * ((visible)? 1 : 0)
            fillMode: Image.PreserveAspectFit


            smooth: true

            anchors.left: parent.left
            anchors.leftMargin: 8 * (visible? 2 : 1) * (contentFillsCompletely? 0 : 1)
            anchors.verticalCenter: parent.verticalCenter

            onVisibleChanged: {
                internals.resetContent();
            }

            onStatusChanged: {
                if (status == Image.Error) {
                    source = mapAssetsPath + "images/layout/poiGeneric.png";
                    imageUrl = source;
                } else {
                    //
                }
            }

            MouseArea {
                anchors.fill: parent

                onPressed: {
                    bubbleBody.state = "pressed";
                    bubblePeak.state = "pressed";

                    infoBubble.fingerDownOnInfoBubble = true;
                }

                onReleased: {
                    infoBubble.fingerDownOnInfoBubble = false;
                }

                onExited: {
                    bubbleBody.state = "default";
                    bubblePeak.state = "default";
                }

                onDoubleClicked: {
                    singleClickLazyEmitter.stop();
                    infoBubble.infoBubbleDoubleClicked();
                }

                onClicked: {
                    singleClickLazyEmitter.start();
                }

                onPressAndHold: {
                    infoBubble.infoBubbleLongPressed();
                }
            }
        }

        Rectangle {
            id: bubbleBodyContent

            clip: true

            anchors.left: bubbleIcon.right
            anchors.leftMargin: 8 * (contentFillsCompletely? 0 : 1)
            anchors.right: parent.right
            anchors.rightMargin: 16 * (contentFillsCompletely? 0 : 1)
            anchors.top: parent.top
            anchors.topMargin: 14 * (contentFillsCompletely? 0 : 1)
            anchors.bottom: parent.bottom
            anchors.bottomMargin: 14 * (contentFillsCompletely? 0 : 1)

            color: "transparent"

            MouseArea {
                anchors.fill: parent

                onPressed: {
                    if (clickEffectEnabled) {
                        bubbleBody.state = "pressed";
                        bubblePeak.state = "pressed";

                        infoBubble.fingerDownOnInfoBubble = true;
                    }
                }

                onReleased: {
                    infoBubble.fingerDownOnInfoBubble = false;
                }

                onExited: {
                    bubbleBody.state = "default";
                    bubblePeak.state = "default";
                }

                onDoubleClicked: {
                    singleClickLazyEmitter.stop();
                    infoBubble.infoBubbleDoubleClicked();
                }

                onClicked: {
                    singleClickLazyEmitter.start();
                }

                onPressAndHold: {
                    infoBubble.infoBubbleLongPressed();
                }
            }
        }

        states: [
            State {
                name: "default"
                PropertyChanges {
                    target: hoverRectangle

                    visible: false
                }
            },
            State {
                name: "pressed"
                PropertyChanges {
                    target: hoverRectangle

                    visible: true
                }
            }
        ]
    }

    Item {
        width: bubbleBody.width / 2
        height: 40

        opacity: bubblePeak.opacity

        anchors.horizontalCenter: bubbleBody.horizontalCenter
        anchors.top: bubbleBody.bottom

        MouseArea {
            anchors.fill: parent;
            onClicked: { infoBubble.infoBubbleClosed() }
        }
    }

    Item {
        id: bubblePeak

        clip: true
        opacity: 0

        width: 17
        height: 29

        anchors.horizontalCenter: bubbleBody.horizontalCenter
        anchors.top: bubbleBody.bottom

        Image {
            id: bubblePeakImage

            source: mapAssetsPath + "images/layout/infobubbleDrop.png"

            anchors.top: parent.top

            states: [
                State {
                    name: "default"
                    PropertyChanges {
                        target: bubblePeakImage

                        anchors.topMargin: 0
                    }
                },
                State {
                    name: "pressed"
                    PropertyChanges {
                        target: bubblePeakImage

                        anchors.topMargin: -30
                    }
                }
            ]
        }

        property alias state: bubblePeakImage.state

        MouseArea {
            anchors.fill: parent;
            onClicked: { infoBubble.infoBubbleClosed() }
        }
    }

    Timer {
        running: infoBubble.open && bubbleBodyContent.children.length <= 1
        interval: 1000 * 5
        onTriggered: {
            infoBubble.open = false;
        }
    }

    Timer {
        id: singleClickLazyEmitter

        interval: 250

        onTriggered: {
            infoBubble.infoBubbleClicked();
        }
    }

    Connections {
        target: infoBubble.content
        ignoreUnknownSignals: true

        onWidthChanged: {
            internals.resetContent();
        }
        onHeightChanged: {
            internals.resetContent();
        }
    }
}
