import QtQuick 1.0
import MapsPlugin 1.0

import "MapApi.js" as MapApi

Item {
    id: mapEventHandler

    property variant mapView
    property variant longPressRectangle

    property int fingerPointerSquareWidthInPixels: 25
    property int oneLoopDuration: 10
    property int singleClickDurationInLoops: 20
    property int doubleClickWaitingDurationInLoops: 20
    property int doubleFingerSlideDurationInLoops: 999999999
    property int longPressDurationInLoops: 35
    property int longPressPixelTolerance: 10
    property int twoFingersSlideMinimumDeltaX: 30
    property int twoFingersSlideMinimumDeltaY: 30

    Item {
        id: internals

        property bool mousePressed: false
        property bool mouseDragged: false
        property bool twoFingersPressed: false
        property bool singleClick : false
        property variant clickedGeoPos: GeoCoordinates {}
        property variant clickedPixelPos: null
        property variant twoFingersMouseDownPosition: null
        property int twoFingersSlideTotalDeltaX: 0
        property int twoFingersSlideTotalDeltaY: 0
        property bool twoFingersSlideEnabled: false
        property variant lastDragCenter: GeoCoordinates {}
        property bool zoomHappened: false
        property int zoomScaleOnTwoFingersDown: 0
    }

    function distanceToPressCenter() {
        if (internals.clickedPixelPos && internals.clickedPixelPos.x && internals.clickedPixelPos.y) {
            var pixel = mapView.geoToScreen(internals.clickedGeoPos, true);
            var dx = pixel.x - internals.clickedPixelPos.x;
            var dy = pixel.y - internals.clickedPixelPos.y;
            return Math.sqrt(dx * dx + dy * dy);
        } else {
            return longPressPixelTolerance + 1;
        }
    }

    function pixelDistanceFromGeoToPixel(geo, pixel) {
        var pixel1 = mapView.geoToScreen(geo, true);
        var dx = pixel1.x - pixel.x;
        var dy = pixel1.y - pixel.y;
        return Math.sqrt(dx * dx + dy * dy);
    }

    Connections {
        target: mapView
        ignoreUnknownSignals: true

        onMouseDown: {
            internals.zoomHappened = false;
            if (geoPos) {
                internals.mousePressed = true;

                mouseEventLoop.softStart();

                internals.clickedGeoPos.latitude = geoPos.latitude;
                internals.clickedGeoPos.longitude = geoPos.longitude;

                internals.lastDragCenter.latitude  = mapView.center.latitude;
                internals.lastDragCenter.longitude = mapView.center.longitude;

                internals.clickedPixelPos  = pixelPos;
            }
        }

        onMouseUp: {
            if (mouseEventLoop.running && mouseEventLoop.counter <= mapEventHandler.singleClickDurationInLoops) {
                internals.singleClick = true;
                mouseEventLoop.dbcounter = 0;
            }

            if (internals.mouseDragged) {
                mapView.mapDragEnded(pixelPos, geoPos);
                internals.mouseDragged = false;
            }

            internals.mousePressed = false;

            if (!internals.singleClick) {
                mouseEventLoop.stop();
            }

            mapView.mapReleased(pixelPos, geoPos);
        }

        onMouseDoubleClick: {
            internals.zoomHappened = false;
            internals.singleClick = false;
            mouseEventLoop.reset();

            if (!lazyRestoreTolerance.toHappen) {
                mapView.mouseMoveTolerance *= 10;
                lazyRestoreTolerance.toHappen = true;
            }
            lazyRestoreTolerance.start();

            mapView.mapDoubleClicked(pixelPos, geoPos);
        }

        onTwoFingersMouseDown: {
            internals.twoFingersPressed = true;
            internals.twoFingersMouseDownPosition = null;

            internals.zoomHappened = false;
            internals.zoomScaleOnTwoFingersDown = mapView.zoomScale;

            twoFingerEventLoop.restart();
            twoFingerSlideEventLoop.start();
            internals.twoFingersSlideTotalDeltaX = 0;
            internals.twoFingersSlideTotalDeltaY = 0;
            internals.twoFingersSlideEnabled = false;
        }

        onTwoFingersMouseUp: {
            if (!internals.zoomHappened && twoFingerEventLoop.running) {
                if (!mapView.blockInteractivity && !mapView.getBehavior(mapView._NO_INTERACTIVITY_)) {
                    mapView.mapTwoFingerDoubleClicked(pixelPos, geoPos)
                }
                twoFingerEventLoop.stop();
            }

            internals.twoFingersPressed = false;
            twoFingerSlideEventLoop.stop();
            internals.twoFingersSlideTotalDeltaX = 0;
            internals.twoFingersSlideTotalDeltaY = 0;
            internals.twoFingersSlideEnabled = false;
        }

        onTwoFingersMove: {
            if (!mapView.blockInteractivity && !mapView.getBehavior(mapView._NO_INTERACTIVITY_)) {
                if (true || twoFingerSlideEventLoop.running) {
                    var deltaX = internals.twoFingersMouseDownPosition? internals.twoFingersMouseDownPosition.x - centerPos.x : 0;
                    var deltaY = internals.twoFingersMouseDownPosition? internals.twoFingersMouseDownPosition.y - centerPos.y : 0;

                    internals.twoFingersSlideTotalDeltaX += deltaX;
                    internals.twoFingersSlideTotalDeltaY += deltaY;

                    if (!internals.twoFingersSlideEnabled && Math.abs(internals.twoFingersSlideTotalDeltaY) >= twoFingersSlideMinimumDeltaY) {
                        internals.twoFingersSlideEnabled = true;
                    }

                    if (internals.twoFingersSlideEnabled) {
                        if (Math.abs(deltaX) < 45 && Math.abs(deltaY) < 45) {
                            mapView.twoFingerSlide(-1 * deltaX, -1 * deltaY);
                        }
                    }

                    internals.twoFingersMouseDownPosition = centerPos;
                }
            }
        }

        onZoomScaleChanged: {
            if (mapView.zoomScale != internals.zoomScaleOnTwoFingersDown) {
                internals.zoomHappened = true;
            }
        }

        //onCenterChanged: {
        onMouseMove: {
            if (internals.mousePressed) {
                if (!internals.mouseDragged && /*mouseEventLoop.running && */distanceToPressCenter() > longPressPixelTolerance) {
                    mapView.mapDragStarted(mapView.geoToScreen(mapView.center, false), mapView.center);
                    internals.mouseDragged = true;
                    mouseEventLoop.running = false;
                } else if (mouseEventLoop.running) {
                    var pixel = mapView.geoToScreen(mapView.center, false);
                    var distance = pixelDistanceFromGeoToPixel(internals.lastDragCenter, pixel);

                    if (distance > mapView.farDragPixelDistance) {
                        mapView.mapDraggedFar(pixel, mapView.center);
                        internals.lastDragCenter.latitude  = mapView.center.latitude;
                        internals.lastDragCenter.longitude = mapView.center.longitude;
                    }

                    mapView.mapDragged(pixel, mapView.center);
                }
            }
        }
    }

    Connections {
        target: longPressRectangle
        ignoreUnknownSignals: true

        onCompleted: { mapView.mapLongPressRectangleCompleted(internals.clickedPixelPos, internals.clickedGeoPos); internals.mouseDragged = false; }
        onCanceled: { mapView.mapLongPressRectangleCanceled(internals.clickedPixelPos, internals.clickedGeoPos); internals.mouseDragged = false; }
    }

    Timer {
        id: lazyRestoreTolerance

        interval: 250
        property bool toHappen: false

        onTriggered: {
            mapView.mouseMoveTolerance /= 10;
            toHappen = false;
        }
    }

    Timer {
        id: twoFingerEventLoop

        repeat: false
        running: false
        interval: 2 * oneLoopDuration * doubleClickWaitingDurationInLoops

        onTriggered: {
            //twoFingerEventLoop.stop();
        }
    }

    Timer {
        id: twoFingerSlideEventLoop

        repeat: false
        running: false
        interval: oneLoopDuration * doubleFingerSlideDurationInLoops

        onTriggered: {}
    }

    Timer {
        id: mouseEventLoop

        repeat: true
        running: false
        interval: oneLoopDuration

        property int counter: 0
        property int dbcounter: 0

        function softStart() {
            mouseEventLoop.counter = 0;
            mouseEventLoop.start();
        }

        function reset() {
            mouseEventLoop.stop();
            mouseEventLoop.counter = 0;
            mouseEventLoop.dbcounter = 0;
        }

        onTriggered: {
            mouseEventLoop.counter++;

            if (internals.singleClick) {
                mouseEventLoop.dbcounter++;
                if (mouseEventLoop.dbcounter > doubleClickWaitingDurationInLoops) {
                    mouseEventLoop.stop();

                    internals.singleClick = false;

                    if (!internals.zoomHappened) {
                        var rectangle = Qt.rect(
                            internals.clickedPixelPos.x - fingerPointerSquareWidthInPixels / 2,
                            internals.clickedPixelPos.y - fingerPointerSquareWidthInPixels / 2,
                            fingerPointerSquareWidthInPixels,
                            fingerPointerSquareWidthInPixels
                        );
                        var mapObjects = mapView.getMapObjectsIn(rectangle);
                        var mapObject = null;

                        if (mapObjects.length >= 0) {
                            for (var i = 0; i < mapObjects.length; i++) {
                                //fix problem in plugin for device -- always has 1 item with type == 2
                                //the problem is with "broken" polygons - they seem to be everywhere
                                if (mapObjects[i].type != 2) {
                                    mapObject = mapObjects[i];
                                    break;
                                }
                            }
                        }

                        if (mapObject && MapApi.checkIfClickedOnCurrentPosition(internals.clickedPixelPos)) {
                            mapView.mapCurrentLocationClicked(internals.clickedPixelPos, internals.clickedGeoPos, mapObject);
                        } else {
                            mapView.mapClicked(internals.clickedPixelPos, internals.clickedGeoPos, mapObject);

                            if (mapObject) {
                                if (mapObject.type == MapObject.TRANSITSTOP && mapView.getBehavior(mapView._NO_PUBLIC_TRANSPORT_STOPS_CLICK_)) {
                                    // skip signal emiting at all
                                } else {
                                    mapView.mapObjectsSelected(mapObject);
                                    mapView.model.selectObject(mapObject, false);
                                }
                            }
                        }
                    }
                }
            } else if (mouseEventLoop.counter > longPressDurationInLoops) {
                mouseEventLoop.stop();

                if (distanceToPressCenter() <= longPressPixelTolerance && !internals.twoFingersPressed) {
                    if (!mapView.getBehavior(mapView._NO_LONG_PRESS_AT_ALL_)) {
                        mapView.model.selectObject({type: "longPress", position: mapView.center});
                        mapView.mapLongPressed(internals.clickedPixelPos, internals.clickedGeoPos);
                    }
                }
            }
        }
    }
}
