import QtQuick 1.0
import "Persistence.js" as C

Item {
    id: self

    property string dbName: "NokiaMapsDataBase"
    property string dbVersion: "1.0"
    property string dbComment: "Application settings storage"
    property int    dbLimit: 1000000
    property bool   dbReady: false;

    Item {
        id: internals
        property bool   _isRunning: false
        property int    _maxRevisionNumber: 10;

        function _addReq(id, params) {
            C._queue.push({
                'id': id,
                'params': params
            });
        }

        function _dequeueReq() {
            return C._queue.shift();
        }

        function _getDBConnection() {
            return openDatabaseSync(dbName, dbVersion, dbComment, dbLimit);
        }

        function _ensureDatabase(conn) {        
            conn.executeSql('CREATE TABLE IF NOT EXISTS Revisions(name TEXT, date TEXT, msec TEXT)');
            conn.executeSql('CREATE TABLE IF NOT EXISTS Properties(key TEXT, value TEXT, revision TEXT)');
            dbReady = true;
        }

        function _startQueue() {
            if (!_isRunning) {
                _isRunning = true;

                _getDBConnection().transaction(function(conn) {
                    try {
                        if (!dbReady) {
                            _ensureDatabase(conn);
                        }
                        while (C._queue.length > 0) {
                            _handleRequest(_dequeueReq(), conn);
                        }
                        //_printDB(conn);
                        _isRunning = false;
                    }
                    catch (e) {
                        log.info("cannot do action in DB : " + e);
                    }
                });
            }
        }

        function _set(conn, key, value, revision) {
            var rs = conn.executeSql("SELECT * FROM Properties WHERE key=? AND revision=?", [key, revision]);
            if (rs.rows.length > 0) {
                conn.executeSql("UPDATE Properties SET value=? WHERE key=? AND revision=?", [value, key, revision]);
            }
            else {
                conn.executeSql("INSERT INTO Properties VALUES(?, ?, ?)", [key, value, revision]);
            }
        }

        function _setRevision(conn, map, revision) {
            for (var key in map) {
                _set(conn, key, map[key], revision);
            }
        }

        function _get(conn, key, revision, callback) {
            var rs = conn.executeSql("SELECT * FROM Properties WHERE key=? AND revision=?", [key, revision]);

            if (rs.rows.length > 0)
                callback(rs.rows.item(0).value);
            else
                callback(null);
        }

        function _getRevision(conn, revision, callback) {
            var rs = conn.executeSql("SELECT * FROM Properties WHERE revision=?", [revision]);
            if (rs.rows.length > 0) {
                var results = {};
                var row;

                for (var i = 0, l = rs.rows.length; i < l; i++) {
                    row = rs.rows.item(i);
                    results[row.key] = row.value;
                }
                callback(results);
            }
            else {
                callback(null);
            }
        }

        function _remove(conn, key, revision) {
            conn.executeSql("DELETE FROM Properties WHERE key=? AND revision=?", [key, revision]);
            var rs = conn.executeSql("SELECT * FROM Properties WHERE revision=?", [revision]);
            if (rs.rows.length === 0) {
                conn.executeSql("DELETE FROM Revisions WHERE name=?", [revision]);
            }
        }

        function _removeRevision(conn, revision) {
            conn.executeSql("DELETE FROM Revisions WHERE name=?", [revision]);
            conn.executeSql("DELETE FROM Properties WHERE revision=?", [revision]);
        }

        function _removeAll(conn) {
            conn.executeSql('DELETE FROM Revisions');
            conn.executeSql('DELETE FROM Properties');
        }

        function _handleRequest(req, conn) {
            var p  = req.params;
            var id = req.id;            

            switch(id) {
                case C.REQ_SET:     _set(conn, p.key, p.value, p.revision);     break;
                case C.REQ_SET_REV: _setRevision(conn, p.map, p.revision);      break;
                case C.REQ_GET:     _get(conn, p.key, p.revision, p.callback);  break;
                case C.REQ_GET_REV: _getRevision(conn, p.revision, p.callback); break;
                case C.REQ_DEL:     _remove(conn, p.key, p.revision);           break;
                case C.REQ_DEL_REV: _removeRevision(conn, p.revision);          break;
                case C.REQ_DEL_ALL: _removeAll(conn);                           break;
            }

            if (p.revision) {
                _updateRevision(p.revision, conn);

                if (id == C.REQ_SET || id == C.REQ_SET_REV) {
                    _removeOldRevisions(conn);
                }
            }
        }

        function _removeOldRevisions(conn) {
            // remove old revisions
            var rs = conn.executeSql("SELECT * FROM Revisions");
            var l = rs.rows.length;
            if (l > _maxRevisionNumber) {
                var revs = [];
                for (var i = 0; i < l; i++) {
                    revs.push(rs.rows.item(i));
                }
                revs.sort(function(a, b) {
                    return parseInt(b.msec) - parseInt(a.msec);
                });
                for (var i = _maxRevisionNumber; i < l; i++) {
                    _removeRevision(conn, revs[i].name);
                }
            }
        }

        function _updateRevision(revision, conn) {
            var date = (new Date());
            var sdate = date.toString();
            var msecs = date.getMilliseconds();
            var rs = conn.executeSql('SELECT * FROM Revisions WHERE name=?', [revision]);

            if (rs.rows.length > 0) {
                conn.executeSql('UPDATE Revisions SET date=?, msec=? WHERE name=?', [sdate, msecs, revision ]);
            }
            else {
                conn.executeSql('INSERT INTO Revisions VALUES(?, ?, ?)', [revision, sdate, msecs]);
            }
        }

        function _printDB(conn) {
            log.trace("+++++REVISIONS+++++++");
            var rs = conn.executeSql('SELECT * FROM Revisions');
            for (var i = 0; i < rs.rows.length; i++) {
                log.trace("Name: " + rs.rows.item(i).name + ", Date: " + rs.rows.item(i).date);
            }

            log.trace("+++++PROPERTIES+++++++");
            var rs = conn.executeSql('SELECT * FROM Properties');
            for (var i = 0; i < rs.rows.length; i++) {
                log.trace("Key: " + rs.rows.item(i).key + ", Value: " + rs.rows.item(i).value + ", Rev: " + rs.rows.item(i).revision);
            }
        }
    }


    function set(key, value, revision) {
        internals._addReq(C.REQ_SET, {
            key: key,
            value: value,
            revision: revision
        });
        internals._startQueue();
    }

    function setRevision(map, revision) {
        internals._addReq(C.REQ_SET_REV, {
            map: map,
            revision: revision
        });
        internals._startQueue();
    }

    function get(key, revision, callback) {
        internals._addReq(C.REQ_GET, {
            key: key,
            revision: revision,
            callback: callback
        });
        internals._startQueue();
    }

    function getRevision(revision, callback) {
        internals._addReq(C.REQ_GET_REV, {
            revision: revision,
            callback: callback
        });
        internals._startQueue();
    }

    function remove(key, revision) {
        internals._addReq(C.REQ_DEL, {
            key: key,
            revision: revision
        });
        internals._startQueue();
    }

    function removeRevision(revision) {
        internals._addReq(C.REQ_DEL_REV, {
            revision: revision
        });
        internals._startQueue();
    }

    function removeAll() {
        internals._addReq(C.REQ_DEL_ALL, {});
        internals._startQueue();
    }
}
