import QtQuick 1.0

import com.nokia.meego 1.0
import com.nokia.extras 1.0

import MapsPlugin 1.0 as MapsPlugin

import ovi.connector.places 1.0

import "ui"
import "AudioSettings.js" as Logic

Page {
    id: audioSettings;
    anchors.fill: parent;
    clip: true;

    signal voiceChosen(int voiceId)
    signal voiceDownloadStarted(int index)

    property int chosenIndex: -1
    property int currentVoiceId: -1
    property alias model: voiceModel
    property bool cancelled: false
    property bool updating: false
    /* the voice catalog plugin object */
    property QtObject __voiceCatalog

    onUpdatingChanged: {
        if(updating) {
            Logic.disableAll(voiceModel, chosenIndex);
            log.trace("[audio settings]: updating index " + chosenIndex + ", disabling all others");
        } else {
            Logic.enableAll(voiceModel);
            log.trace("[audio settings]: updating done, enabling all");
        }
    }

    tools: ToolBarLayout {
        visible: false;
        ToolIcon {
            iconId: "icon-m-toolbar-back"
            objectName: "toolBarIconBackButton"
            anchors.left: parent.left
            onClicked: {
                //cancelled = true;
                //__voiceCatalog.cancelVoiceDownload();
                //Logic.saveData(db);
                //voiceChosen(audioSettings.currentVoiceId);
                pageStack.pop();
            }
        }
    }

    Rectangle {
        anchors.fill: parent
        color: "#f0f1f2"
    }

    SettingsPersistence { id: db }

    ListModel { id: voiceModel }

    ListView {
        id: audioList
        objectName: "audioList"
        anchors {
            left: parent.left
            right: parent.right
            top: header.bottom
            bottom: parent.bottom
        }
        clip: true;
        model: voiceModel
        delegate: VoiceCatalogItem {
            downloadProgress: progress
            downloadSize: voiceSize

            isInstalled: installed
            updateVoice: update
            radioChecked: selectedVoice
            isEnabled: itemEnabled
            error: errorMessage || false
            loading: isLoading || false

            name: voiceName
            gender: voiceGender

            queryDialog: updateDialog

            onDownloadVoiceClicked: { Logic.downloadVoice(index, voiceModel, downloadStartTimer); }
            onCancelDownloadClicked: { Logic.stopDownload(true, index, voiceModel, downloadStartTimer); }
            onUpdateVoiceClicked: { Logic.updateVoice(index, voiceModel, downloadStartTimer); }
            onVoiceItemChosen: { Logic.selectVoice(index, voiceModel); }
        }

        QueryDialog {
            id: updateDialog

            titleText: qsTranslate("Maps", "qtn_navigationconnector_voicecatalogdelegate_updatedialog_title")
            message: qsTranslate("Maps", "qtn_navigationconnector_voicecatalogdelegate_updatedialog_message")
            acceptButtonText: qsTranslate("Maps", "qtn_navigationconnector_voicecatalogdelegate_updatedialog_accept")
            rejectButtonText: qsTranslate("Maps", "qtn_navigationconnector_voicecatalogdelegate_updatedialog_reject")

            onAccepted: { Logic.updateVoice(index, voiceModel, downloadStartTimer); }
            onRejected: { Logic.selectVoice(index, voiceModel); }

            property int index
        }
    }
    ScrollDecorator {
        flickableItem: audioList
    }

    Header {
        id: header
        text: qsTranslate("Maps", "qtn_navigationconnector_audiosettings_heading")
        anchors {
            left: parent.left
            right: parent.right
        }
        dropShadow: false
    }

    InfoBanner {
        id: errorBanner
        text: qsTranslate("Maps", "qtn_navigationconnector_audiosettings_error")
        timerEnabled: true
        timerShowTime: 3 * 1000   // 3 seconds
    }

    Timer {
        id: downloadStartTimer
        interval: 60*1000 // 60 seconds
        onTriggered: { Logic.stopDownload(false); errorBanner.show(); }
    }

    Connections {
        target: __voiceCatalog
        onDownloadDone: {
            if (success && chosenIndex !== -1) {
//                currentVoiceId = voiceModel.get(chosenIndex).id;
                Logic.selectVoice(chosenIndex, voiceModel, voiceModel.get(chosenIndex).id);
                voiceModel.setProperty(chosenIndex, "update", 0);
            } else if (!cancelled && chosenIndex !== -1) {
                errorBanner.show();
                cancelled = false;
            }
            chosenIndex = -1;
            Logic.updateCatalogList(__voiceCatalog.catalogList, __voiceCatalog.localVoiceSkins, voiceModel, currentVoiceId, updating, chosenIndex);
        }

        onDownloadProgressChanged: {
            if(downloadStartTimer.running) {
                downloadStartTimer.stop();
            }
            if(chosenIndex !== -1) {
                voiceModel.setProperty(chosenIndex, "progress", __voiceCatalog.downloadProgress);
            }
        }

        onCatalogListChanged: {
            Logic.updateCatalogList(__voiceCatalog.catalogList, __voiceCatalog.localVoiceSkins, voiceModel, currentVoiceId, updating, chosenIndex);
        }

        onLocalVoiceSkinsChanged: {
            Logic.updateCatalogList(__voiceCatalog.catalogList, __voiceCatalog.localVoiceSkins, voiceModel, currentVoiceId, updating, chosenIndex);
        }
    }

    function updateVoiceCatalog() {
        log.trace("updating voice catalog");
        appState.sessionOpened.disconnect(updateVoiceCatalog);
        var catalogList = __voiceCatalog.catalogList;
        if (!catalogList || catalogList.length === 0) {
            voiceModel.clear();
            voiceModel.append(Logic.getVoiceItem(false, -1, {}, true, false));
            if (!mainscreen.onlineMode) {
                appState.openSession();
                appState.sessionOpened.connect(updateVoiceCatalog);
                appState.sessionError.connect(Logic.popOnError);
                return;
            } else {
                appState.sessionError.disconnect(Logic.popOnError);
            }

            if (!__voiceCatalog.downloadCatalog())
                log.warning("unable to download voice catalog");
        } else {
            Logic.loadData(db);
            Logic.updateCatalogList(catalogList, __voiceCatalog.localVoiceSkins, voiceModel, currentVoiceId, updating, chosenIndex);
        }
    }

    Component.onCompleted: {
        updateVoiceCatalog();
    }
}
