import QtQuick 1.0

import com.nokia.meego 1.0

import MapsPlugin 1.0

import ovi.connector 1.0
import ovi.connector.map 1.0

import "ui"
import "GuidanceViewer.js" as Logic
import "RoutingUtils.js" as RoutingUtils

// See http://bf-hudson-00.europe.nokia.com:8080/job/maps-plugin-api/javadoc/a00011.html
ConnectorPage {
    id: guidancePage

    property int __defaultGuidanceZoomScale: map.zoomLevels[15] //taken from AVALON-2877
    property int __beepsAndVibrationsVoiceSkinId: 1003
    property bool __keepGuidanceRunning: false

    property bool compassStarted: false
    property bool resumeCompass: false

    GuidanceToolBarLayout {
        id: guidanceToolbar

        guidance: guidance
        isLandscape: guidancePage.width > guidancePage.height
        mode: "buttons"
        toolbar: guidancePage.pageStack ? guidancePage.pageStack.toolBar : undefined

        onButtonLayoutTimeoutReached: {
            if (!Logic.destinationReached && mode == "buttons") {
                mode = "info";
            }
        }

        onStopClicked: {
            lazyGuidanceStartTimer.stop();
            Logic.endGuidance();
        }

        onOverviewClicked: {
            isInOverviewMode = !isInOverviewMode;
            Logic.showOverview(isInOverviewMode);
        }

        onViewMenuClicked: { guidanceViewMenu.open(); }

        onBackClicked: {
            Logic.endGuidance();
        }
    }

    tools: guidanceToolbar.getLayout(guidanceToolbar.mode)

    SettingsPersistence { id: db }

    VoiceCatalog {
        id: voiceCat
        onDownloadDone: { __vcIsDownloading = false; }
    }

    property int __vcDownloadingIndex: -1
    property bool __vcIsDownloading: false

    Component {
        id: audioSettings

        AudioSettings {
            __voiceCatalog: voiceCat
            chosenIndex: __vcDownloadingIndex
            updating: __vcIsDownloading
            onVoiceChosen: {
                if(voiceId !== -1 && guidanceViewMenu.audioChecked && guidanceViewMenu.audioEnabled) {
                    guidance.voiceSkinId = voiceId;
                } else {
                    guidance.voiceSkinId = 0;
                }
            }
            onVoiceDownloadStarted: {
                __vcIsDownloading = true;
                __vcDownloadingIndex = index;
            }
        }
    }

    GuidanceViewMenu {
        id: guidanceViewMenu

        visualParent: mainscreen
        onAudioSettingsClicked: {
            __keepGuidanceRunning = true;
            pageStack.push(audioSettings);
        }
        onBacklightSettingChanged: {
            mainscreen.backlightsAlwaysOn = checked;
            appState.disableScreenSaver(checked);
        }

        onStatusChanged: {
            if(status === DialogStatus.Opening) {
                Logic.loadSettings();
            } else if(status === DialogStatus.Closing) {
                db.set("audioCommandsOn", guidanceViewMenu.audioChecked);
                Logic.loadSettings();
            }
        }
    }

    Guidance {
        id: guidance
        distanceUnit: appState.locale.measurementSystem === "imperial" ? Guidance.IMPERIAL : Guidance.METRIC
        property int selectedVoiceSkinId: 0

        onDestinationReached: {
            if (typeof (analyticsComponent) != "undefined") {
                var currentTimeStamp = new Date();
                var guidanceDuration;
                analyticsComponent.timeStampEndNavigation = currentTimeStamp.getTime();
                guidanceDuration = analyticsComponent.timeStampEndNavigation - analyticsComponent.timeStampStartNavigation;
                analyticsComponent.logIMEIActivity("EndNavigation", { duration: guidanceDuration/1000 + " sec", reason: "reachedDestination" } );
            }
            //map.removeWaypointIcons();

            Logic.showOverview(false);
            guidanceToolbar.mode = "finished";
            guidanceHeader.showDestinationReached(/*model.waypoints[1]*/);
            Logic.destinationReached = true;
            guidanceToolbar.isInOverviewMode = false;
        }

        onDestinationDistanceChanged: {
            if (destinationDistance < 20) {
                destinationReached();
            }
        }

        onManeuverEvent: {
            guidanceHeader.updateHeader(guidance.nextManeuver, guidance.nextManeuverDistance);
        }

        onRouteUpdated: {
            map.route.showWaypointIcons();
            guidanceHeader.showRouteUpdated();
        }

        onGuidanceUpdated: {
            guidanceToolbar.updateGuidance(guidance, map.averageSpeed);
            guidanceHeader.updateHeader(guidance.nextManeuver, guidance.nextManeuverDistance);
        }

        Component.onCompleted: {
            guidance.showOnMap(map);
            Logic.loadSettings();

            guidanceHeader.showJustStarted();
        }

        mapUpdateMode: {
            return Logic.getGuidanceUpdateMode(true);
        }
    }

    onStatusChanged: {
        if (status == PageStatus.Activating) {
            if (!__keepGuidanceRunning) {
                log.info('Trying to calculate route:');
                guidancePage.model.calculationDone.connect(Logic.onRouteCalculationDone);
                Logic.startNavigation();
            }
            __keepGuidanceRunning = false;
            guidanceToolbar.toolbarTimer.start();
            Logic.loadSettings();
            appState.disableScreenSaver(mainscreen.backlightsAlwaysOn);
        } else if (status === PageStatus.Deactivating) {
            if (!__keepGuidanceRunning) {
                map.removeRouteFromMap();
                guidancePage.model.calculationDone.disconnect(Logic.onRouteCalculationDone);
                waitFor30SecondsAfterPositionLost.running = false;
                timerForMoreThan30Seconds.running = false;
            }
            guidanceToolbar.toolbarTimer.stop();
            appState.disableScreenSaver(false);
        }
    }

    GuidanceHeader {
        id: guidanceHeader
        anchors.top: parent.top
        guidance: guidance
        showRepeatButton: guidanceViewMenu.audioChecked && guidance.voiceSkinId !== __beepsAndVibrationsVoiceSkinId

        visible: !guidanceToolbar.isInOverviewMode

        onRepeatButtonClicked: {
            if (!justIgnoreSomeoneClickingTooFastTheRepeatButton.running) {
                guidance.repeat();
                justIgnoreSomeoneClickingTooFastTheRepeatButton.start();

                log.trace("*** apply voice command...")
            } else {
                log.trace("*** skip voice command...")

            }
        }
    }

    RouteHeader {
        id: routeHeader
        anchors.top: parent.top

        mapView: map
        route: guidancePage.model
        state: parent.state

        visible: guidanceToolbar.isInOverviewMode
        onClicked: Logic.showOverview(true)
    }

    Rectangle {
        anchors.top: guidanceHeader.bottom

        height: 4
        width: parent.width
        gradient: Gradient {
            GradientStop { position: 0.0; color: "black" }
            GradientStop { position: 1.0; color: "transparent" }
        }
        opacity: 0.5
        z: 1
    }

    property RouteViewer __map: map
    RouteViewer {
        id: map
        mainScreen: mainscreen

        runInGuidance: true

        anchors { top: guidanceHeader.bottom; bottom: parent.bottom; left: parent.left; right: parent.right; }
        behaviors: "_NO_LONG_PRESS_AT_ALL_:true;_POSITION_TYPE_REQUIRED_GPS_:true;_NO_INFO_BUBBLE_DEFAULT_LONG_PRESS_ACTION_:true;_NO_INFO_BUBBLE_DEFAULT_OBJECT_PRESS_ACTION_:true;_NO_INFO_BUBBLE_CLICK_EFFECT_:true;_NO_MY_POSITION_DEFAULT_CLICK_ACTION_:true;_NO_COMPASS_ALLOWED_:true;"
        visible: true
        attachedToLocation: true

        route: parent.model

        onMapDragged: Logic.setMapFollowsPosition(false)
        onMapClicked: Logic.setMapFollowsPosition(false)
        onMyPositionBarClicked: {
            if (!guidanceToolbar.isInOverviewMode) {
                Logic.setMapFollowsPosition(true, true);
            } else {
                map.doFlyToCurrentPosition();
            }
            guidanceToolbar.isInOverviewMode = false;
        }

        onAnimationDone: { Logic.handleAnimationDone() }

        onDeviceOrientationModeChanged: {
            if (guidanceToolbar.isInOverviewMode && route && route.calculated) {
                lazyZoomToRoute.start();
            }
        }
    }

    QueryDialog {
        id: guidanceRoutingDialog
        objectName: "guidanceRoutingDialog"

        property bool forceCompleteClose: false
        property bool dontClearRoute: false

        function closeComplete() {
            close();
            lazyOpener.stop();
        }

        function openWithContent(title, message, buttonText, forceClose, dontClear) {
            guidanceRoutingDialog.forceCompleteClose = !!forceClose;
            guidanceRoutingDialog.dontClearRoute = !!dontClear;
            guidanceRoutingDialog.titleText = title || " ";
            guidanceRoutingDialog.message = message;
            guidanceRoutingDialogButton.text = buttonText;
            //guidanceRoutingDialog.open();
            lazyOpener.start();
        }

        title: BusyIndicator {
            anchors {
                horizontalCenter: parent.horizontalCenter
            }
            height: 60

            running: true
            platformStyle: BusyIndicatorStyle {inverted: true}
        }
        Timer {
            id: lazyOpener
            interval: 250 * 2
            onTriggered: {
                guidanceRoutingDialog.open();
            }
        }

        buttons: Row {
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
            anchors.verticalCenterOffset: 20
            Column {
                spacing: 10

                Rectangle { height: 50 }
                Button {
                    id: guidanceRoutingDialogButton

                    platformStyle: ButtonStyle {
                        inverted: true
                    }

                    onClicked: guidanceRoutingDialog.accept()
                }
            }
        }

        function cancelAndClose(aCompleteClose) {
            map.currentPositionChanged.disconnect(Logic.startNavigation);
            if (aCompleteClose) {
                if (!dontClearRoute) {
                    model.clearRoute(true);
                }
                Logic.endGuidance("-cancel");
            } else {
                model.cancelCalculation();
                lazyGuidanceStartTimer.stop();
                pageStack.pop(null, true);
            }
            forceCompleteClose = false;
            dontClearRoute = false;
        }

        onAccepted: cancelAndClose(forceCompleteClose)
        onRejected: cancelAndClose(forceCompleteClose)
    }

    Units {
        id: units
    }

    ReverseGeoCodeFinder {
        id: finder

        mapView: map

        property string actionToPerform

        onFindComplete: {
            if (actionToPerform == "logAnalyticsStartFromCurrentPosition") {
                Logic.doLogStartNavigation(context.headingText, context.bodyText);
            }

            actionToPerform = "";
        }
    }

    Connections {
        target: mainscreen
        ignoreUnknownSignals: true

        onUseMagicCompassInGuidanceInstanadOfMOSChanged: {
            if (guidance.running) {
                if (mainscreen.useMagicCompassInGuidanceInstanadOfMOS) {
                    Logic.startCompass();
                } else {
                    Logic.stopCompass();
                }

                guidance.mapUpdateMode = Logic.getGuidanceUpdateMode(map.attachedToLocation);
            }
        }
    }

    Connections {
        target: map
        ignoreUnknownSignals: true

        onPositionTypeChanged: {
            if (map.positionType != map._POSITION_TYPE_GPS_) {
                waitFor30SecondsAfterPositionLost.running = true;
            } else {
                waitFor30SecondsAfterPositionLost.running = false;
                timerForMoreThan30Seconds.running = false;
                map.showRightPopupMessage("");
            }
        }
        onAttachedToLocationChanged: {
            if (!map.attachedToLocation) {
                if (compassStarted) {
                    Logic.stopCompass();

                    resumeCompass = true;
                }
            }
        }

        onMyPositionBarClicked: {
            if (resumeCompass) {
                Logic.startCompass();

                resumeCompass = false;
            }
        }
    }

    Connections {
        target: MapsPlugin
        ignoreUnknownSignals: true

        onDiskcacheWillUnmount: {
            if(guidance.running) {
                guidance.stop();
                guidance.clear();
                guidance.voiceSkinId = 0;
            }
        }
    }

    Connections {
        target: appState
        ignoreUnknownSignals: true

        onMinimized: {
        }
        onMaximized: {
        }
        onWindowDeactivated: {
            if (guidancePage.visible) {
                appState.disableScreenSaver(false);

                if (compassStarted) {
                    appState.stopCompass();
                }
            }
        }
        onWindowActivated: {
            if (guidancePage.visible) {
                appState.disableScreenSaver(mainscreen.backlightsAlwaysOn);

                if (compassStarted) {
                    appState.startCompass();
                }
            }
        }

        onCompassReading: {
            if (compassStarted) {
                compassTimeApplier.setToAverage(azimuth);
            }
        }
    }

    Timer {
        id: justIgnoreSomeoneClickingTooFastTheRepeatButton
        interval: 1000 * 5
    }

    Timer {
        id: lazyZoomToRoute
        interval: 50
        onTriggered: { map.zoomToRoute(false) }
    }

    Timer {
        id: compassTimeApplier

        property real azimuth

        property variant queue
        property variant _MAXIMUM_QUEUE_COUNT_: 7

        function setToAverage(azimuth) {
            var tmp = compassTimeApplier.queue;
            if (!tmp || Math.abs(compassTimeApplier.azimuth - azimuth) > 300) {
                tmp = [];
            }
            tmp.push(azimuth);
            if (tmp.length > compassTimeApplier._MAXIMUM_QUEUE_COUNT_) {
                tmp.shift();
            }
            compassTimeApplier.queue = tmp;

            var sum = 0;
            for (var i = 0; i < tmp.length; i++) {
                sum += tmp[i];
            }

            var newAverage = sum / tmp.length;
            compassTimeApplier.azimuth = sum / tmp.length;
        }

        running: mainscreen.magicCompass && mainscreen.useMagicCompassInGuidanceInstanadOfMOS && map.visible && mainscreen.applicationMaximized && compassStarted
        repeat: true
        interval: 25
        onTriggered: {
            if (azimuth) {
                if (map.attachedToLocation) {
                    map.orientation = compassTimeApplier.azimuth;
                }
            }
        }
    }

    Timer {
        id: timerForMoreThan30Seconds
        interval: 1000
        repeat: true

        property int seconds: 0

        function go() {
            timerForMoreThan30Seconds.seconds = 0;
            timerForMoreThan30Seconds.start();
        }

        onTriggered: {
            timerForMoreThan30Seconds.seconds++;
            map.showRightPopupMessage(qsTranslate("Maps", "qtn_navigationconnector_position_lost_for_30_seconds").arg(units.normalizeTime(30 + timerForMoreThan30Seconds.seconds).readableWithUnits));
        }
    }

    Timer {
        id: waitFor30SecondsAfterPositionLost
        interval: 1000 * 30
        onTriggered: {
            timerForMoreThan30Seconds.go();
        }
    }

    /* Timer used while waiting for GPS. */
    Timer {
        id: waitForGPSTimer
        running: false
        interval: 30 * 1000
        onTriggered: guidanceRoutingDialog.accept();
    }

    Timer {
        id: lazyGuidanceStartTimer
        interval: 2 * 1000
        onTriggered: Logic.startGuidance();
    }

    /**
      * Used for development!!! Just to update the guidance while simulating
      */
    Timer {
        interval: 250
        running: !!(Logic.simulationOn && guidance.running)
        repeat: true
        onTriggered: {
            guidanceToolbar.updateGuidance(guidance);
            guidanceHeader.updateHeader(guidance.nextManeuver, guidance.nextManeuverDistance);
            map.stopPositionProvider();
        }
    }


}
