import QtQuick 1.0

import MapsPlugin 1.0 as MapsPlugin
import "RoutingUtils.js" as RoutingUtils

QtObject {
    id: route

    /*! the mode of transportation, one of "drive", "walk" or "pt" */
    property string mode

    /*! the waypoints of the route, an array of {latitude, longitude} or Place */
    property variant waypoints: []

    /*! if set to true automatically calculate the best mode depending on the areal distance between the waypoints */
    property bool autoMode

    /*! further specification if mode is "drive", one of "optimized", "fastest", "shortest" */
    property string driveMode: "optimized"

    /*! sets if ferries can be used in the route, "walk" or "drive" mode only */
    property bool allowFerries: true

    /*! sets if highways can be used in the route, "drive" mode only */
    property bool allowHighways: true

    /*! sets if tunnels can be used in the route, "drive" mode only */
    property bool allowTunnels: true

    /*! sets if motorail trains can be used in the route, "drive" mode only */
    property bool allowMotorailTrains: true

    /*! sets if tollroads can be used in the route, "drive" mode only */
    property bool allowTollRoads: true

    /*! sets if unpaved roads can be used in the route, "drive" mode only */
    property bool allowUnpavedRoads: true

    /*! the route color */
    property color color: "#FF8A235C"

    /*! true if the route has been calculated */
    property bool calculated: !!__calculationResult

    /*! the length (distance to travel) of the route in meters, -1 if the route has not been calculated  */
    property int length: __calculationResult ? __calculationResult.length : -1

    /*! the travel time of the route in seconds, -1 if the route has not been calculated */
    property int duration: __calculationResult ? __calculationResult.duration : -1

    /* the route's maneuvers */
    property list<QtObject> maneuvers

    /*! signal after calculation ended */
    signal calculationDone(int errorCode, int violatedOptions)

    /*! starts a (re-)calculation of the route*/
    function calculate() {
        try {
            var routePlan = RoutingUtils.createRoutePlan(route);
            __router.calculate(routePlan);
        } catch (ex) {
            print(ex.message);
            route.calculationDone(ex.errorCode, 0);
        }
    }

    /*
     *  If a mode is already chosen should return true if the route is over 50000 km
     */
    function isAutoModeRecommended() {
        if (mode !== "") {
            var distance = RoutingUtils.computeDistance(
                    RoutingUtils.createGeoCoordinatesJson(waypoints[0]),
                    RoutingUtils.createGeoCoordinatesJson(waypoints[waypoints.length - 1])
                );
            return distance > 50000
        }

        return true;
    }

    /*! cancels an ongoing calculation of the route*/
    function cancelCalculation() {
        __router.cancel();
    }

    /*! clears the route */
    function clearRoute(aForce) {
        __reset(aForce);
    }

    function __reset(force) {
        if (route.__calculationResult) {
            route.maneuvers = [];
            route.__calculationResult.destroy();
            route.__calculationResult = null;
        }
        if (force) {
            route.autoMode = true;
            route.waypoints = [];
        }
    }

    /*! \ignore the Router, just for internal use */
    property QtObject __router: MapsPlugin.Router {
        onRoutingDone: {
            var error = errorCode;
            if (route.mode === "pt") {
                var validRoute = false;
                if (calculatedRoute) {
                    for (var i=0; !validRoute && i<calculatedRoute.maneuvers.length; i++) {
                        validRoute  = calculatedRoute.maneuvers[i].transitRouteElements.length > 0
                    }
                }
                if (!validRoute) {
                    error = MapsPlugin.Router.ERROR_GRAPH_DISCONNECTED;
                }
            }
            route.__reset();
            if (error === 0) {
                route.__calculationResult = calculatedRoute.clone(mainscreen || route.parent || route);
                route.maneuvers = route.__calculationResult.maneuvers;
            }

            route.calculationDone(error, violatedOptions);
        }
    }

    /*! the route calculation result, type is maps plugin Route, just for internal use! */
    property variant __calculationResult

    Component.onDestruction: {
        __reset();
    }

//    onWaypointsChanged: {print("******************* WAYPOINTS CHANGED"); __reset();}
    onModeChanged: __reset()
    onDriveModeChanged:  __reset()
    onAllowFerriesChanged: __reset()
    onAllowTunnelsChanged: __reset()
    onAllowHighwaysChanged: __reset()
    onAllowMotorailTrainsChanged: __reset()
    onAllowTollRoadsChanged: __reset()
    onAllowUnpavedRoadsChanged: __reset()
}
