var ptWalkAction = 17;
/**
 * Computes the distance between two geo locations in meters.
 *
 * @param {Object} from the first geo location
 * @param {Number} from.longitude the longitude of the geo location.
 * @param {Object} to the second geo location
 * @param {Number} to.latitude the latitude of the geo location.
 * @param {Number} to.longitude the longitude of the geo location.
 * @return {Number} the distance in meters
 */
function computeDistance(from, to) {
    var PI_OVER_180 = Math.PI / 180,
        thetaA = from.latitude * PI_OVER_180,
        phiA = from.longitude * PI_OVER_180,
        cosThetaA = Math.cos(thetaA),
        ax = cosThetaA * Math.sin(phiA),
        ay = cosThetaA * Math.cos(phiA),
        az = Math.sin(thetaA),
        thetaB = to.latitude * PI_OVER_180,
        phiB = to.longitude * PI_OVER_180,
        cosThetaB = Math.cos(thetaB),
        bx = cosThetaB * Math.sin(phiB),
        by = cosThetaB * Math.cos(phiB),
        bz = Math.sin(thetaB);
    return 6371e3 * Math.acos(ax*bx + ay*by + az*bz);
}

function createPluginObject(objectType, parent) {
    return Qt.createQmlObject("import MapsPlugin 1.0; " + objectType + "{}", parent, null);
}

function createObject(objectType, parent) {
    return Qt.createQmlObject(objectType + "{}", parent, null);
}

function createRoutePlan(route) {
    var wps = route.waypoints;
    var len = wps.length;
    if (!wps || len < 2) {
        throw {
            errorCode: MapsPlugin.Router.ERROR_ROUTE_CORRUPTED,
            message: "route.waypoints must contain at least 2 waypoints, " +
                route.waypoints ? "but has " + route.waypoints.length : "is undefined/null."
        };
    }

    var plan = createPluginObject("RoutePlan", route);
    for (var i = 0; i < len; i++) {
        var wp = wps[i];

        if (!wp) {
            throw {
                errorCode: MapsPlugin.Router.ERROR_ROUTE_CORRUPTED,
                message: "Route calculation failed. Waypoint #" + i + " is not defined"
            };
        }

        if ("uri" in wp && "properties" in wp) { // TODO: improve place detection, e.g. by improving the URI
            // place object
            wp = createGeoCoordinates(wp.properties, route);
        } else if (wp.objectName === undefined && typeof wp.latitude === 'number' && typeof wp.longitude === 'number') {
            // JSON object
            wp = createGeoCoordinates(wp, route);
        }

        plan.addStopover(wp);
    }

    var distance = computeDistance(createGeoCoordinatesJson(wps[0]), createGeoCoordinatesJson(wps[len - 1]));

    if (route.autoMode) {
        if (distance <= 3000) {
            route.mode = "walk";
        } else {
            route.mode = "drive";
        }
        route.autoMode = false;
    }

    if (route.mode === "walk" && distance > 50000) {
        throw {
            errorCode: MapsPlugin.Router.ERROR_CANNOT_DO_PEDESTRIAN,
            message: "Route calculation failed. Distance too long for pedestrian route."
        };
    }

    var options = createPluginObject("RouteOptions", route);
    if (route.mode === "drive") {
        options.setPresetMode(MapsPlugin.RouteOptions.PRESET_CAR_ECONOMIC);
        options.routeMode = MapsPlugin.RouteOptions.MODE_CAR;
        options.routeType = MapsPlugin.RouteOptions.TYPE_ECONOMIC;

        options.allowTollRoads = true;
        options.allowHighways = true;
        options.allowFerries = true;
        options.allowRailFerries = true;
        options.allowTunnels = true;
        options.allowDirtRoads = false;
        options.allowParks = false;
        options.allowPublicTransport = false;
    } else if (route.mode === "walk") {
        options.setPresetMode(MapsPlugin.RouteOptions.PRESET_PEDESTRIAN);
        options.routeMode = MapsPlugin.RouteOptions.MODE_PEDESTRIAN;
        options.routeType = MapsPlugin.RouteOptions.TYPE_SHORTEST;

        options.allowTollRoads = true;
        options.allowHighways = false;
        options.allowFerries = true;
        options.allowRailFerries = false;
        options.allowTunnels = true;
        options.allowDirtRoads = true;
        options.allowParks = true;
        options.allowPublicTransport = false;
    } else if (route.mode === "pt") {
        options.setPresetMode(MapsPlugin.RouteOptions.PRESET_PUBLIC_TRANSPORT);
        options.routeMode = MapsPlugin.RouteOptions.MODE_PEDESTRIAN;
        options.routeType = MapsPlugin.RouteOptions.TYPE_SHORTEST;

        options.allowTollRoads = true;
        options.allowHighways = false;
        options.allowFerries = true;
        options.allowRailFerries = false;
        options.allowTunnels = true;
        options.allowDirtRoads = true;
        options.allowParks = true;
        options.allowPublicTransport = true;
    } else {
        throw {
            errorCode: MapsPlugin.Router.ERROR_ROUTE_CORRUPTED ,
            message: "Route calculation failed. Unknown route mode '" + route.mode + "'"
        };
    }

    plan.setAllRouteOptions(options);
    options.destroy();

    return plan;
}

function createGeoCoordinatesJson(obj) {
    if (obj.properties) {
        obj = obj.properties;
    }

    return {
        latitude: obj.latitude === undefined ? obj.geoLatitude : obj.latitude,
        longitude: obj.longitude === undefined ? obj.geoLongitude : obj.longitude
    }
}

function getRouteMode() {
    if (typeof(route.mode) != "undefined" && route.mode != "" ) {
        return route.mode;
    }
    return "N/A";
}

function createGeoCoordinates(obj, parent) {
    var geoCoords = createPluginObject("GeoCoordinates", parent);
    var geoCoordsJson = createGeoCoordinatesJson(obj);
    geoCoords.latitude = geoCoordsJson.latitude;
    geoCoords.longitude = geoCoordsJson.longitude;
    return geoCoords;
}


function getErrorText(errorCode) {
    if (errorCode === MapsPlugin.Router.ERROR_NONE) {
        return "";
    } else if (errorCode === MapsPlugin.Router.ERROR_GRAPH_DISCONNECTED_CHECK_OPTIONS ||
               errorCode === MapsPlugin.Router.ERROR_NO_END_POINT_CHECK_OPTIONS ||
               errorCode === MapsPlugin.Router.ERROR_ROUTE_USES_DISABLED_ROADS) {
        return qsTranslate("Maps", "qtn_navigationconnector_routing_error_check_options");// "No route found, some option(for example disabled highways) may be prohibiting it.";
    } else if (errorCode === MapsPlugin.Router.ERROR_NO_START_POINT) {
        return qsTranslate("Maps", "qtn_navigationconnector_routing_error_invalid_start");
    } else if (errorCode === MapsPlugin.Router.ERROR_NO_END_POINT) {
        return qsTranslate("Maps", "qtn_navigationconnector_routing_error_invalid_destination");
    } else if (errorCode === MapsPlugin.Router.ERROR_CANNOT_DO_PEDESTRIAN) {
        var measurementSystem;
        try {
            measurementSystem = appState.locale.measurementSystem;
        } catch (e) {
        }
        if(measurementSystem === "imperial") {
            return qsTranslate("Maps", "qtn_navigationconnector_routing_error_pedestrian_too_long_mi");
        }
        return qsTranslate("Maps", "qtn_navigationconnector_routing_error_pedestrian_too_long_km");
    } else if (errorCode === MapsPlugin.Router.ERROR_ROUTE_CORRUPTED ||
               errorCode === MapsPlugin.Router.ERROR_OUT_OF_MEMORY ||
               errorCode === MapsPlugin.Router.ERROR_GRAPH_DISCONNECTED) {
        return qsTranslate("Maps", "qtn_navigationconnector_routing_error_calculation_failed");
    }

    return qsTranslate("Maps", "qtn_navigationconnector_routing_error_unknown %1").arg(errorCode) // "Unknown routing error #" + errorCode;
}

function compressPtManeuvers(aRoute) {
    // Not defined by plugin
    var enterStationAction = 15;
    var exitStationAction = 16;

    var segment = [];
    var newManeuvers= [];
    var n, nManeuver;
    for (var i=0; i<aRoute.maneuvers.length; i++) {
        if (aRoute.maneuvers[i].action === enterStationAction) {
            var ptStartElement = aRoute.maneuvers[i].transitRouteElements[0];
            var currentPtSegmentLine = ptStartElement.lineName;
            n = i+1;
            segment = [ptStartElement];
            while (n < aRoute.maneuvers.length && aRoute.maneuvers[n].action === enterStationAction) {
                nManeuver = aRoute.maneuvers[n];
                var ptElement = nManeuver.transitRouteElements[0];
                if (currentPtSegmentLine === ptElement.lineName) {
                    segment.push(ptElement);
                    n++;
                } else {
                    break;
                }
            }
            newManeuvers.push(createPtSegment(segment, true));
            i = n - 1;
        } else {
            var walkLength = aRoute.maneuvers[i].distanceFromPreviousManeuver,
                summaryDirection = "";
            n = i+1;
            segment = [aRoute.maneuvers[i]];

            while (n < aRoute.maneuvers.length && aRoute.maneuvers[n].action !== enterStationAction) {
                segment.push(aRoute.maneuvers[n]);
                walkLength += aRoute.maneuvers[n].distanceFromPreviousManeuver;
                n++;
            }
            if(n < aRoute.maneuvers.length && aRoute.maneuvers[n].action === enterStationAction) {
                summaryDirection = aRoute.maneuvers[n].transitRouteElements[0].departureStation.name;
            } else {
                summaryDirection = formatShortAddress(aRoute.waypoints[aRoute.waypoints.length - 1].properties);
                if(summaryDirection === "") {
                    var tmp = (n === aRoute.maneuvers.length ? n-1 : n);
                    summaryDirection = aRoute.maneuvers[tmp].streetName || qsTranslate("Maps", "qtn_navigationconnector_routing_destination");
                }
            }

            // insert the summary at the beginning
            var summary = {
                action: 100,
                destination: summaryDirection,
                length: walkLength,
                icon: 64
            }
            // Insert the summary. If it's the first maneuver, get rid of the start flag maneuver, otherwise just insert
            segment.splice(0, (i===0? 1 : 0), summary);

            if(i === 0) {
                // The start flag, unique only to the start of the route
                segment.splice(0, 0, aRoute.maneuvers[i]);
            }
            newManeuvers.push(createWalkSegment(segment, aRoute.maneuvers[i].geoCoordinates));
            i = n - 1;
        }
    }
    return newManeuvers;
}

function createPtSegment(aSegment, isEnterManeuver) {
    var enterStationAction = 15,
        exitStationAction = 16,
        firstElement = aSegment[0],
        lastElement = aSegment[aSegment.length - 1],
        travelTime = 0,
        duration = 0,
        stationStops = []
    ;

    for(var i=0; isEnterManeuver && i<aSegment.length; i++) {
        travelTime += aSegment[i].travelTime;
        duration += aSegment[i].duration;
        stationStops.push(copyPluginStation(aSegment[i].departureStation));
    }

    // [ace]: note: always add the arrival station, otherwise we get one less stop and the exit information is wrong.
    stationStops.push(copyPluginStation(lastElement.arrivalStation));

    return {
        geoCoordinates: copyPluginCoordinates((isEnterManeuver ? firstElement.departureStation : lastElement.arrivalStation).egressCoordinates),
        action: isEnterManeuver ? enterStationAction : exitStationAction,
        type: "pt",
        roadElements: {},
        lineName: firstElement.lineName,
        destination: firstElement.destination,
        id: firstElement.id.stringId,
        primaryLineColor: lastElement.primaryLineColor,
        numStops: aSegment.length-1,
        stationStops: stationStops,
        systemLogo: copyPluginLogo(firstElement.systemLogo),
        departureStation: copyPluginStation(firstElement.departureStation),
        arrivalStation: copyPluginStation(lastElement.arrivalStation),
        travelTime: travelTime,
        duration: duration,
        departureTime: firstElement.departureTime,
        arrivalTime: lastElement.arrivalTime,
        transitType: firstElement.transitType,
        transitTypeName: firstElement.transitTypeName
    }
}

function createWalkSegment(aSegment, aCoords) {
    return {
        geoCoordinates: copyPluginCoordinates(/*aSegment[0].geoCoordinates*/aCoords),
        action: ptWalkAction,
        walkManeuvers: aSegment
    }
}
function copyPluginStation(aStation) {
    return {
            "id": aStation.id,
            "name": aStation.name,
            "platformLevel": aStation.platformLevel,
            "platformCoordinates": copyPluginCoordinates(aStation.platformCoordinates),
            "egressCoordinates": copyPluginCoordinates(aStation.egressCoordinates)
    }
}

function copyPluginCoordinates(aCoordinates) {
    return {
            "latitude": aCoordinates.latitude,
            "longitude": aCoordinates.longitude,
            "altitude": aCoordinates.altitude,
            "timeOffset": aCoordinates.timeOffset
        }
}

function copyPluginLogo(aLogo) {
    return {
            "category": aLogo ? aLogo.category : null,
            "svgContent": aLogo ? aLogo.svgContent : null,
            "localUrl": aLogo ? aLogo.localUrl : null,
            "isValid": aLogo ? aLogo.isValid : null,
            "size": aLogo ? aLogo.size : null,
            "imageUrl": aLogo ? aLogo.imageUrl : null
    }
}

function formatShortAddress(properties) {
    if (!properties) {
        return '';
    }

    var line = (properties.addrStreetName || "") + " "  + (properties.addrHouseNumber || "");
    return line.replace(/ /g,"").length>0?line:"";
}

