/**
 * Address formating data utility, works on LBSP object structure
 * @namespace
 */

 .pragma library

var addressFormatter = (function () {
    var isChina = false;

    function setChina(_isChina) {
        isChina = _isChina? true : false;
        invalidateAddressMap();
    }

    /**
     * Address map containing displaye templates
     * @private
     */
    var addressMap = [];
    function invalidateAddressMap() {
        addressMap = [
            {
                codes: {
                    USA: 1,
                    CAN: 1,
                    NZL: 1,
                    IND: 1,
                    GBR: 1,
                    AUS: 1,
                    IRL: 1,
                    LKA: 1
                },
                template: {
                    full: ["f:street", ", ", "f:locality", ", ", "a:countryName"],
                    street: ["a:houseNumber", " ", "a:street"],
                    locality: ["a:district", ", ", "a:city", " ", "a:postalCode"]
                }
            },
            {
                codes: {
                    FRA: 1
                },
                template: {
                    full: ["f:street", ", ", "f:locality", ", ", "a:countryName"],
                    street: ["a:houseNumber", " ", "a:street"],
                    locality: ["a:city", " ", "a:postalCode"]
                }
            },
            {
                codes: {
                    CHN: 1,
                    HKG: 1,
                    TWN: 1
                },
                template: {
                    full: ["a:countryName", " ", "f:locality", isChina? "" : " ", "a:district", isChina? "" : " ", "f:street", ", ", "a:postalCode"],
                    street: ["a:street", isChina? "" : " ", "a:houseNumber"],
                    locality: ["a:city"]
                }
            }
        ];
    }
    invalidateAddressMap();

    /**
     * Address default format template used for not specified address templates
     * @private
     * @static
     */
    var defaultTemplate = {
        full: ["f:street", ", ", "f:locality", ", ", "a:countryName"],
        street: ["a:street", " ", "a:houseNumber"],
        locality: ["a:postalCode", " ", "a:city"]
    };
    
    /**
     * Function selects the proper address tempalte according to
     * country code
     * 
     * @param {Object} countryCode
     * @private
     * @static
     */
    function getAddressTemplate(countryCode) {
        for (var i = 0, l = addressMap.length; i < l; i++){
            var map = addressMap[i];
            if (map.codes[countryCode]) {
                return map.template;
            }        
        }
        
        return defaultTemplate;
    }
    
    /**
     * Function formats address should be called within the current template scope
     * 
     * @param {Object} templateItem
     * @param {Object} address
     */
    function formatItem(key, template, address) {
        var templateItem = template[key],
            result = [],
            value,
            attrKey,
            elem,
            lastSeparator = null,
            setSeparator = false;
         
        for (var i = 0, l = templateItem.length; i < l; i++) {
            elem = templateItem[i];
            attrKey = elem.split(":");
            if (attrKey.length === 2) { //function or attribute not separator
                value = (attrKey[0] === "a")? address[attrKey[1]] : formatItem(attrKey[1], template, address);
                if (value) {
                    if (lastSeparator) {
                        result.push(lastSeparator);
                        lastSeparator = null;
                    }
                    result.push(value);
                    setSeparator = true;
                }
                
            } else {
                //separator
                if (setSeparator) {
                    lastSeparator = elem;
                }
                setSeparator = false;
            }    
        }
        
        return result.join("");
    }
    
    /**
     * Function returns full formatted address
     * @param {Object} address 
     */
    function formattedAddress(address) {
        if (address) {
            var template = getAddressTemplate(address.country);
            return formatItem("full", template, address);
        } else {
            return "";
        }
    }
    
    /**
     * Function returns the address lines i.e formatted street name 
     * and formatted localty
     * @param {Object} address
     */
    function addressLines(address) {
        if (address) {
            var template = getAddressTemplate(address.country), result = {
                street: formatItem("street", template, address),
                locality: formatItem("locality", template, address),
                country: address.countryName
            };
            
            return result;
        }
        
        return {};
    }
    
    /**
     * Function returns a formatted address structure for returned LBSP place/location object
     * @param {Object} addressObject
     * @return {String} formatted address object
     * 
     * @example
     *     var formattedAddress = addressFormatter.getLocationFormattedAddress(data);
     */
    function getLocationFormattedAddress(addressObject) {
        if (addressObject) {
            return formattedAddress(addressObject);
        } else {
            return "";
        }
    }
    
    /**
     * Function return address lines for the place object
     * @param {Object} placeObject (see also {@link ovi.places.objects.Place})
     * @return {Object} the address lines object which containe formatted street, locality and country name
     */
    function getLocationAddressLines(addressObject) {
        if (addressObject) {
            return addressLines(addressObject);
        } else {
            return {};
        }
    }
    
    return {
        isChina: setChina,
        getAddressTemplate: getAddressTemplate,
        addressLines: addressLines,
        formattedAddress: formattedAddress,
        getLocationFormattedAddress: getLocationFormattedAddress,
        getLocationAddressLines: getLocationAddressLines
    };
    
}());
