import QtQuick 1.0

import com.nokia.meego 1.0

Item {
    id: previewImage

    property string imageUrl
    property int imageIndex
    property real imageWidth
    property real imageHeight

    property string provider: ""
    property bool adContent

    property Image currentItem: image
    property Image previousItem
    property Image nextItem
    property ListModel listModel

    Item {
        id: internals

        property int initialContainerAreaX
        property int initialMouseX
        property bool mouseDownMode: false
        property bool swipeHappened: false
        property bool anySwipeAtAll: false
        property int _MINIMAL_SWIPE_TOLLERANCE_: 10
        property int _MINIMAL_CLOSE_TOLLERANCE_: 3
        property int _DEFAULT_IMAGE_WIDTH_: mainscreen.width - 140
        property int _DEFAULT_IMAGE_HEIGHT_: mainscreen.height - 70

        function replaceImage() {
            currentItem.width = previewImage.imageWidth? previewImage.imageWidth : internals._DEFAULT_IMAGE_WIDTH_;
            currentItem.height = previewImage.imageHeight? previewImage.imageHeight : internals._DEFAULT_IMAGE_HEIGHT_;

            currentItem.source = unescape(imageUrl);
            currentItem.anchors.centerIn = imageCanvas;

            internals.destroyPrevAndNextImages();
            internals.repositionPrevAndNextImages();
        }

        function setImageSize(currentItem) {
            if (currentItem) {
                var ratio = currentItem.sourceSize.width / currentItem.sourceSize.height;
                var maxWidth = Math.min(currentItem.sourceSize.width, imageCanvas.width);
                var maxHeight = Math.min(currentItem.sourceSize.height, imageCanvas.height);
                var maxRatio = maxWidth / maxHeight;

                if (previewImage.imageWidth && previewImage.imageHeight && currentItem == previewImage.currentItem && previewImage.imageWidth != internals._DEFAULT_IMAGE_WIDTH_ && previewImage.imageHeight != internals._DEFAULT_IMAGE_HEIGHT_) {
                    currentItem.width = previewImage.imageWidth;
                    currentItem.height = previewImage.imageHeight;
                } else if (maxRatio < ratio) {
                    currentItem.width = maxWidth;
                    currentItem.height = maxWidth * (1 / ratio);
                } else {
                    currentItem.height = maxHeight;
                    currentItem.width = maxHeight * ratio;
                }

                currentItem.y = (mainscreen.height - currentItem.height - imagePreviewText.paintedHeight - 50) / 2;
            }
        }

        function getNextIndex() {
            var result;

            if (listModel && listModel.count > 0) {
                if (imageIndex < listModel.count - 1) {
                    result = imageIndex + 1;
                } else {
                    result = 0;
                }
            } else {
                result = -1;
            }

            return result;
        }

        function getPreviousIndex() {
            var result;

            if (listModel && listModel.count > 0) {
                if (imageIndex > 0) {
                    result = imageIndex - 1;
                } else {
                    result = listModel.count - 1;
                }
            } else {
                result = -1;
            }

            return result;
        }

        function repositionPrevAndNextImages() {
            var previousImageIndex = internals.getPreviousIndex();
            if (previousImageIndex >= 0) {
                if (previousItem) {
                    previousItem.visible = false;
                    previousItem.destroy();
                }
                previousItem = dummyImage.createObject(imageCanvas);
                previousItem.source = unescape(listModel.get(previousImageIndex).URL);

                previousItem.width = internals._DEFAULT_IMAGE_WIDTH_;
                previousItem.height = internals._DEFAULT_IMAGE_HEIGHT_;
                previousItem.anchors.right = currentItem.left;
                previousItem.anchors.rightMargin = 30;

                if (previousItem.status == Image.Ready) {
                    internals.setImageSize(previousItem);
                }

                //previousItem.visible = true;
            }

            var nextImageIndex = internals.getNextIndex();
            if (nextImageIndex >= 0) {
                if (nextItem) {
                    nextItem.visible = false;
                    nextItem.destroy();
                }
                nextItem = dummyImage.createObject(imageCanvas);
                nextItem.source = unescape(listModel.get(nextImageIndex).URL);

                nextItem.width = internals._DEFAULT_IMAGE_WIDTH_;
                nextItem.height = internals._DEFAULT_IMAGE_HEIGHT_;
                nextItem.anchors.left = currentItem.right;
                nextItem.anchors.leftMargin = 30;

                if (nextItem.status == Image.Ready) {
                    internals.setImageSize(nextItem);
                }

                //nextItem.visible = true;
            }
        }

        function destroyPrevAndNextImages() {
            if (previousItem) {
                previousItem.visible = false;
                previousItem.destroy();
            }
            if (nextItem) {
                nextItem.visible = false;
                nextItem.destroy();
            }
        }

        function prepareForSwipe(mouseX, mouseY) {
            internals.mouseDownMode = true;
            internals.swipeHappened = false;
            internals.anySwipeAtAll = false;

            internals.initialContainerAreaX = currentItem.x;
            var obj = internals.mapToItem(mainscreen, mouseX, mouseY);
            internals.initialMouseX = obj.x;
        }

        function prepareForRelease(mouseX, mouseY) {
            var obj = internals.mapToItem(mainscreen, mouseX, mouseY);
            var delta = obj.x - internals.initialMouseX;

            if (Math.abs(delta) > currentItem.width / 4) {
                if (delta > 0) {
                    if (previousItem) {
                        smoothAnimation.go(1, currentItem.width, internals.getPreviousIndex(), previousItem.source, previousItem.width, previousItem.height);
                    }
                } else if (delta < 0) {
                    if (nextItem) {
                        smoothAnimation.go(-1, -currentItem.width, internals.getNextIndex(), nextItem.source, nextItem.width, nextItem.height);
                    }
                } else {
                    currentItem.anchors.horizontalCenterOffset = 0;
                    internals.swipeHappened = false;
                }
            } else {
                currentItem.anchors.horizontalCenterOffset = 0;
                internals.swipeHappened = false;
            }

            internals.mouseDownMode = false;
        }

        function handleMousePositionChanged(mouseX, mouseY) {
            if (internals.mouseDownMode) {
                var obj = internals.mapToItem(mainscreen, mouseX, mouseY);
                var delta = obj.x - internals.initialMouseX;
                if (Math.abs(delta) > internals._MINIMAL_SWIPE_TOLLERANCE_) {
                    internals.swipeHappened = true;
                }
                if (Math.abs(delta) > internals._MINIMAL_CLOSE_TOLLERANCE_) {
                    internals.anySwipeAtAll = true;
                }

                if (Math.abs(delta) > 10 && Math.abs(delta) < containerArea.width) {
                    if (delta > 0 && nextItem || delta < 0 && previousItem) {
                        currentItem.anchors.horizontalCenterOffset = delta;
                    }
                }
            }
        }

        Timer {
            id: smoothAnimation

            interval: 10
            repeat: true

            property int step: 10
            property int direction
            property int destination
            property int newIndex
            property string newSource
            property real newWidth
            property real newHeight
            property int loop: 0

            function go(direction, destination, newIndex, newSource, newWidth, newHeight) {
                smoothAnimation.loop = 0;
                smoothAnimation.direction = direction;
                smoothAnimation.destination = destination;
                smoothAnimation.newIndex = newIndex;
                smoothAnimation.newSource = newSource;
                smoothAnimation.newWidth = newWidth;
                smoothAnimation.newHeight = newHeight;
                smoothAnimation.start();
            }

            onTriggered: {
                if (Math.abs(currentItem.anchors.horizontalCenterOffset) < Math.abs(destination)) {
                    var nextStep = step + (++loop * 10) * direction;
                    nextStep = (Math.abs(currentItem.anchors.horizontalCenterOffset + nextStep) > Math.abs(destination))? direction * (Math.abs(destination) - Math.abs(currentItem.anchors.horizontalCenterOffset)) : nextStep;
                    currentItem.anchors.horizontalCenterOffset += nextStep;
                } else {
                    smoothAnimation.stop();

                    currentItem.anchors.horizontalCenterOffset = 0;
                    imageWidth = newWidth;
                    imageHeight = newHeight;

                    imageIndex = newIndex;
                    imageUrl = newSource;

                    internals.swipeHappened = false;
                    internals.anySwipeAtAll = false;
                }
            }
        }
    }

    Rectangle {
        id: imagePreviewBg
        color: "#000"
        opacity: Math.min(previewImage.opacity,0.9)
        anchors.fill: previewImage
        MouseArea {
            anchors.fill:parent
            onClicked: {
                if (!internals.anySwipeAtAll) {
                    previewImage.opacity = 0;
                }
            }
            onPressed: {
                internals.prepareForSwipe(mouseX, mouseY);
            }
            onReleased: {
                internals.prepareForRelease(mouseX, mouseY);
            }
            onMousePositionChanged: {
                internals.handleMousePositionChanged(mouseX, mouseY);
            }
        }
    }

    Item {
        id: imageCanvas

        anchors.fill: previewImage
        anchors.topMargin: 35
        anchors.leftMargin: 10
        anchors.rightMargin: 10
        anchors.bottomMargin: 30
        onHeightChanged: {
            internals.setImageSize(currentItem);
        }

        Text {
            id: loadingText
            objectName: "imageLoading"
            color: "#fff"
            font { bold:true;  family: mainscreen.__labelStyle.fontFamily; pixelSize: 22 }
            anchors.centerIn: parent
            anchors.fill: parent
            verticalAlignment: Text.AlignVCenter
            horizontalAlignment: Text.AlignHCenter
            text: qsTranslate("Maps", "qtn_placesconnector_thumbnail_loadingerror");//"Loading\nimage...":"Error\nloading\nimage"
            visible: currentItem.status == Image.Error
        }

        BusyIndicator {
            id: indicator
            z: 2
            platformStyle: BusyIndicatorStyle { size: "large";  inverted:true }
            running: (currentItem.status == Image.Ready || currentItem.status == Image.Error)? false : true
            visible: !!running
            anchors.centerIn: parent
        }

        Image {
            id: image
            opacity: previewImage.opacity
            anchors.centerIn: parent
            source: unescape(imageUrl)
            onStatusChanged: {
                if (status == Image.Ready) {
                    internals.setImageSize(currentItem);
                }
            }
            onSourceChanged: {
                if (status == Image.Ready) {
                    internals.setImageSize(currentItem);
                }
            }
        }

        Text {
            id: imagePreviewText
            objectName: "imagePreviewText"
            text: {
                if (provider) {
                    return (qsTranslate("Maps", "qtn_placesconnector_source") + ": " + provider);
                } else if (adContent) {
                    return qsTranslate("Maps", "qtn_placesconnector_viewer_from_owner_image"); // source: business owner
                } else {
                    return  "";
                }
            }
            visible: !indicator.visible

            elide: Text.ElideRight
            opacity: previewImage.opacity
            color: "#fff"
            anchors.top: currentItem.bottom
            anchors.topMargin: 10
            x: currentItem.x
            width: parent.width

            font.pixelSize: 16
            font.family: mainscreen.__labelStyle.fontFamily
        }
    }

    onImageUrlChanged: {
        internals.replaceImage();
    }

    Behavior on opacity {
        NumberAnimation { }
    }

    Component {
        id: dummyImage

        Image {
            id: self
            //opacity: previewImage.opacity

            visible: internals.swipeHappened
            Behavior on visible { PropertyAnimation { target: self; property: "opacity"; from: 0; to: 1; duration: 250; } }

            onStatusChanged: {
                if (status == Image.Ready) {
                    internals.setImageSize(self);
                }
            }
            onSourceChanged: {
                if (status == Image.Ready) {
                    internals.setImageSize(self);
                }
            }

            Text {
                id: loadingText
                objectName: "imageLoading"
                color: "#fff"
                font { bold:true;  family: mainscreen.__labelStyle.fontFamily; pixelSize: 22 }
                anchors.centerIn: parent
                anchors.fill: parent
                verticalAlignment: Text.AlignVCenter
                horizontalAlignment: Text.AlignHCenter
                text: qsTranslate("Maps", "qtn_placesconnector_thumbnail_loadingerror");//"Loading\nimage...":"Error\nloading\nimage"
                visible: self.status == Image.Error
            }

            BusyIndicator {
                platformStyle: BusyIndicatorStyle { size: "large";  inverted: true }
                running: (self.status == Image.Ready || self.status == Image.Error)? false : true
                visible: !!running && !indicator.visible
                anchors.centerIn: parent
            }
        }
    }
}
