import QtQuick 1.0

import com.nokia.meego 1.0

import ovi.connector 1.0
import ovi.connector.places 1.0

Item {
    id: componentRoot
    anchors {left: parent.left; right: parent.right }
    height: content.height

    signal selectNearbyPlace(variant object)
    signal zoomToTop()

    property alias title: titleHeading.headerText
    property alias message: noDataItemText.text
    property alias model: list.model

    property bool showFullList: false
    property bool modelReady: (list.model !== undefined && list.model.ready)
    property int numToShow: 2
    property int modelCount: modelReady ? list.model.count : 0

    visible: {
        if (list.model !== undefined) {
            if (list.model.ready) {
                return (list.model.visualProperties.showNoResults || modelCount !== 0);
            } else {
                return list.model.visualProperties.showLoading;
            }
        } else {
            return false;
        }
    }

    Column {
        id: content
        height: titleHeading.height +
                (loadingItem.visible ? loadingItem.height : 0) +
                (list.model.count > 0 ? list.height : 0) +
                endButtonContainer.height

        anchors {left: parent.left; right: parent.right }
        OviListHeader {
            id: titleHeading
            headerText: model ? model.categoryName : ""
        }

        PlacesCommonItemDelegate {
            id: loadingItem
            height: Math.max(noDataItemText.height, 105)
            anchors { left: parent.left; right: parent.right}
            visible: !modelReady || (modelReady && list.model.count < 1 && list.model.message !== "")
            showSeparator: false
            NearbyItem {
                BusyIndicator {
                    id: busy
                    visible: !modelReady
                    running: true
                    anchors { left: parent.left; leftMargin: 10 }
                    y: parent.height/3;
                }
                MapsText {
                    id: noDataItemText
                    visible: (modelReady && list.model.count < 1 && list.model.message !== "")
                    state: "listSmall"
                    text: parent.visible ? list.model.message : ""
                    anchors { left: parent.left; leftMargin: 10; verticalCenter: parent.verticalCenter; }
                    width: parent.width
                    wrapMode: Text.WordWrap;
                    elide: Text.ElideNone
                }
            }
        }

        ListView {
            id: list
            anchors {
                left: parent.left
                right: parent.right
            }
            clip: true
            interactive: false
            visible: list.model.count > 0
            delegate: PlaceListItem {
                onItemSelected: { selectNearbyPlace(list.model.get(index)); }
                onItemLongPressed: { longPressedItem(index); }
            }

            onHeightChanged: {
                if (height < 1) { height = 105; }
            }
        }

        Item {
            id: endButtonContainer
            anchors {
                left: parent.left
                right: parent.right
            }
            height: endButton.visible? endButton.height + 20 : 0

            Button {
                id: endButton
                anchors.centerIn: parent
                // JUST LEAVE IT LIKE THIS !!!
                // QML cannot re-validate the height of a ListView always and cannot redraw stuff
                // SO to be sure that we will redraw the list above we SHOW this button even if it shall not be visible
                // and make it opacity not visible just to force the re-invalidate of the list
                visible: modelReady && list.model.count >= numToShow
                opacity: modelReady && list.model.count > numToShow? 1 : 0
                text: modelReady ? (showFullList ? list.model.hideString : list.model.moreString) : ""
                onClicked: {
                    componentRoot.state = (componentRoot.state === "opening" ? "closing" : "opening");
                }
            }
        }

    }

    states: [
        State {
            name: "opening"
            PropertyChanges {
                target: list
                height: 105 * list.model.count + 15
            }
            PropertyChanges {
                target: componentRoot
                showFullList: true
            }
        },
        State {
            name: "closing"
            StateChangeScript {
                name: "closeExpandedList"
                script: {
                    list.height = getClosedSectionHeight();
                    componentRoot.showFullList = false;
                }
            }
            StateChangeScript {
                name: "backToTop"
                script: {
                    zoomToTop();
                }
            }
        }
    ]

    Connections {
        target: endButton
        onVisibleChanged: {
            if(state !== "opening") {
                list.height = getClosedSectionHeight();
            }
        }
    }

    transitions: [
        Transition {
            from: "opening"; to: "closing";
            SequentialAnimation {
                ParallelAnimation {
                    ScriptAction { scriptName: "backToTop" }
                    NumberAnimation {target: list; properties: "height"; duration: getHeightAnimationSpeed(); easing.type: Easing.Linear }
                }
                ScriptAction { scriptName: "closeExpandedList" }
            }
        },
        Transition {
            from: "*"; to: "opening";
            PropertyAnimation {properties: "height"; duration: 40 * list.model.count; easing.type: Easing.InQuad}
        }
    ]

    onModelReadyChanged: {
        if (model.message !== "") return;
        if (modelReady) {
            list.height = getClosedSectionHeight();
            showFullList = false;
        }
    }

    onModelCountChanged: {
        state = "";
        showFullList = false;
    }

    function getHeightAnimationSpeed() {
        var speed = 200;
        if (state === "opening") {
            speed = 40 * list.model.count;
        } else if(state === "closing") {
            speed = 80 * list.model.count;
        }
        return speed;
    }

    function getClosedSectionHeight() {
        var result;
        if (list.model.count < numToShow) {
            result = 105 * list.model.count;
        } else {
            result = numToShow * 105;
        }
        return result ? result : 105;
    }
}
